/*
 *  TWIPROM.cpp
 *  Two Wire Interface EEPROM Library
 *  Created by Franky on 07/02/09.
 *  Licensed under LGPL (free to modify and use as you wish)
 */



#include "TWIPROM.h"
#include <Wire.h>

#include "WConstants.h"

ClassTWIPROM TWIPROM;


ClassTWIPROM::ClassTWIPROM() { }

void ClassTWIPROM::begin() {
	Wire.begin();
	return;
}
void ClassTWIPROM::gen_address() {
	device_address = ( (A2 & 0x01)<<2 | (A1 & 0x01)<<1 | (A0 & 0x01) | 0x50);
}
void ClassTWIPROM::assign(byte C,byte B, byte A) {
	pinA0 = A; pinA1 = B; pinA2 = C;
	if (pinA0 < VCC) pinMode(pinA0, OUTPUT);
	if (pinA1 < VCC) pinMode(pinA1, OUTPUT);
	if (pinA2 < VCC) pinMode(pinA2, OUTPUT);
}
void ClassTWIPROM::setBlock(byte B) {
	if (pinA0 == VCC) A0 = 1;
	else if (pinA0 == GND) A0 = 0;
	else { // A0 libre
		A0 = B & 0x01;
		digitalWrite(pinA0, A0);
	}
	if (pinA1 == VCC) A1 = 1;
	else if (pinA1 == GND) A1 = 0;
	else { // A1 libre
		A1 = (B & 0x02)>>1;
		digitalWrite(pinA1, A1);
	}
	if (pinA0 == VCC) A0 = 1;
	else if (pinA0 == GND) A0 = 0;
	else { // A2 libre
		A2 = (B & 0x04)>>2;
		digitalWrite(pinA2, A2);		
	}
	gen_address();
	return;
}

void ClassTWIPROM::write(unsigned int eeaddress, byte data ) {
	uint8_t MSB = (eeaddress >> 8)&0xFF;
	uint8_t LSB = (eeaddress & 0xFF);
	Wire.beginTransmission(device_address);
	Wire.send(MSB); // MSB
	Wire.send(LSB); // LSB
	Wire.send(data);
	Wire.endTransmission();
	delay(5);
	return;
}
byte ClassTWIPROM::read(unsigned int eeaddress ) {
	byte rdata = 0xFF;
	uint8_t MSB = (eeaddress >> 8)&0xFF;
	uint8_t LSB = (eeaddress & 0xFF);
	
	Wire.beginTransmission(device_address);	
	Wire.send(MSB); // MSB
	Wire.send(LSB); // LSB
	Wire.endTransmission();
	asm("nop\n\t");
	Wire.requestFrom((int)device_address,(int)1);
	if (Wire.available()) rdata = Wire.receive();
	return rdata;
}

void ClassTWIPROM::write_buffer(unsigned int eeaddresspage, byte * buffer, int length ) {
	Wire.beginTransmission(device_address);
	Wire.send((int)(eeaddresspage >> 8)); // MSB
	Wire.send((int)(eeaddresspage & 0xFF)); // LSB
	for (byte i=0;i<length;i++) Wire.send(buffer[i]);
	Wire.endTransmission();
	return;
}
void ClassTWIPROM::read_buffer(unsigned int eeaddress, byte * buffer, int length ) {
	/*byte MSB = (eeaddress >> 8)&0xFF;
	byte LSB = (eeaddress & 0xFF);
	Wire.beginTransmission(device_address);
	Wire.send((uint8_t)MSB); // MSB
	Wire.send((uint8_t)LSB); // LSB
	Wire.endTransmission();
	Wire.requestFrom((uint8_t)device_address,(uint8_t)length);
	for (int i=0;i<length;i++) {
		if (Wire.available()) { buffer[i] = Wire.receive(); }
	}
	return;*/
	for (int i=0;i<length;i++) buffer[i] = TWIPROM.read(eeaddress+(unsigned int)i);
	return;
}

